# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import greenbyteapi.models.metadata_field

class SiteWithData(object):

    """Implementation of the 'SiteWithData' model.

    TODO: type model description here.

    Attributes:
        site_id (int): The id of a site.
        title (string): TODO: type description here.
        country (string): TODO: type description here.
        identity (string): TODO: type description here.
        metadata (list of MetadataField): A list of metadata fields and their
            values.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "site_id":'siteId',
        "title":'title',
        "country":'country',
        "identity":'identity',
        "metadata":'metadata'
    }

    def __init__(self,
                 site_id=None,
                 title=None,
                 country=None,
                 identity=None,
                 metadata=None):
        """Constructor for the SiteWithData class"""

        # Initialize members of the class
        self.site_id = site_id
        self.title = title
        self.country = country
        self.identity = identity
        self.metadata = metadata


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        site_id = dictionary.get('siteId')
        title = dictionary.get('title')
        country = dictionary.get('country')
        identity = dictionary.get('identity')
        metadata = None
        if dictionary.get('metadata') != None:
            metadata = list()
            for structure in dictionary.get('metadata'):
                metadata.append(greenbyteapi.models.metadata_field.MetadataField.from_dictionary(structure))

        # Return an object of this model
        return cls(site_id,
                   title,
                   country,
                   identity,
                   metadata)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

class TaskPriorityEnum(object):

    """Implementation of the 'TaskPriority' enum.

    The priority of a task.

    Attributes:
        LOW: TODO: type description here.
        MEDIUM: TODO: type description here.
        HIGH: TODO: type description here.

    """

    LOW = 'low'

    MEDIUM = 'medium'

    HIGH = 'high'




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from greenbyteapi.api_helper import APIHelper


class DataControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(DataControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.data

    # Gets authorized data signals for one or more devices.
    #
    #_🔐 This endpoint requires the **Data** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `datasignals.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_data_signals(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')

        # Perform the API call through the SDK function
        result = self.controller.get_data_signals(device_ids)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"dataSignalId":1,"title":"Wind speed","type":"Wind speed","unit":"m/s","'
            'deviceType":{"deviceTypeId":1,"title":"Turbine"}},{"dataSignalId":5,"title"'
            ':"Power","type":"Power","unit":"kW","deviceType":{"deviceTypeId":1,"title":'
            '"Turbine"}}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Gets authorized data signals for one or more devices.
    #
    #_🔐 This endpoint requires the **Data** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `datasignals.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_data_signals_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')

        # Perform the API call through the SDK function
        result = self.controller.get_data_signals(device_ids)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

    # Gets data for multiple devices and data signals in the given
    #resolution. The timestamps are in the time zone configured in the Greenbyte Platform.
    #Use the useUtc flag to get timestamps in UTC for all resolutions other than daily, weekly, monthly and yearly.
    #
    #_🔐 This endpoint requires the **Data** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `data.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_data(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        data_signal_ids = APIHelper.json_deserialize('[1,5]')
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        use_utc = False
        resolution = '10minute'
        aggregate = 'device'
        aggregate_level = 0
        calculation = 'sum'

        # Perform the API call through the SDK function
        result = self.controller.get_data(device_ids, data_signal_ids, timestamp_start, timestamp_end, use_utc, resolution, aggregate, aggregate_level, calculation)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"aggregate":"device","aggregateId":1,"aggregatePathNames":[],"deviceIds"'
            ':[1],"resolution":"hourly","calculation":"sum","dataSignal":{"dataSignalId"'
            ':1,"title":"Wind speed","unit":"m/s"},"data":{"2020-01-01T00:00:00":6.89,"2'
            '020-01-01T01:00:00":8.33}},{"aggregate":"device","aggregateId":1,"aggregate'
            'PathNames":[],"deviceIds":[1],"resolution":"hourly","calculation":"sum","da'
            'taSignal":{"dataSignalId":5,"title":"Power","unit":"kW"},"data":{"2020-01-0'
            '1T00:00:00":584.33,"2020-01-01T01:00:00":1014}}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Gets data for multiple devices and data signals in the given
    #resolution. The timestamps are in the time zone configured in the Greenbyte Platform.
    #Use the useUtc flag to get timestamps in UTC for all resolutions other than daily, weekly, monthly and yearly.
    #
    #_🔐 This endpoint requires the **Data** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `data.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_data_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        data_signal_ids = APIHelper.json_deserialize('[1,5]')
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        use_utc = False
        resolution = '10minute'
        aggregate = 'device'
        aggregate_level = 0
        calculation = 'sum'

        # Perform the API call through the SDK function
        result = self.controller.get_data(device_ids, data_signal_ids, timestamp_start, timestamp_end, use_utc, resolution, aggregate, aggregate_level, calculation)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

    # Gets the most recent data point for each
    #specified device and data signal. The timestamps are in UTC.
    #
    #_🔐 This endpoint requires the **Data** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `realtimedata.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_real_time_data(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        data_signal_ids = APIHelper.json_deserialize('[1,5]')
        aggregate = 'device'
        aggregate_level = 0
        calculation = 'sum'

        # Perform the API call through the SDK function
        result = self.controller.get_real_time_data(device_ids, data_signal_ids, aggregate, aggregate_level, calculation)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"aggregate":"device","aggregateId":24,"aggregatePathNames":[],"deviceIds'
            '":[24],"calculation":"sum","dataSignal":{"dataSignalId":5,"title":"Power","'
            'unit":"kW"},"data":{"2020-03-17T12:50:02Z":2174}},{"aggregate":"device","ag'
            'gregateId":24,"aggregatePathNames":[],"deviceIds":[24],"calculation":"sum",'
            '"dataSignal":{"dataSignalId":1,"title":"Wind speed","unit":"m/s"},"data":{"'
            '2020-03-17T12:50:02Z":12.2}}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Gets the most recent data point for each
    #specified device and data signal. The timestamps are in UTC.
    #
    #_🔐 This endpoint requires the **Data** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `realtimedata.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_real_time_data_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        data_signal_ids = APIHelper.json_deserialize('[1,5]')
        aggregate = 'device'
        aggregate_level = 0
        calculation = 'sum'

        # Perform the API call through the SDK function
        result = self.controller.get_real_time_data(device_ids, data_signal_ids, aggregate, aggregate_level, calculation)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

    # Gets signal data aggregated per availability contract category.
    #
    #_🔐 This endpoint requires the **Data** and **Statuses** endpoint permissions._
    #
    #_This request can also be made using the POST method, 
    #with a request to `datapercategory.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_data_per_category(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        data_signal_id = 248
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        aggregate = 'device'
        aggregate_level = 0
        category = APIHelper.json_deserialize('["stop"]')

        # Perform the API call through the SDK function
        result = self.controller.get_data_per_category(device_ids, data_signal_id, timestamp_start, timestamp_end, aggregate, aggregate_level, category)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '{"dataSignal":{"dataSignalId":248,"title":"Lost Production (Contractual)",'
            '"unit":"kWh"},"calculation":"sum","data":[{"aggregateId":6,"aggregatePathNa'
            'mes":[],"deviceIds":[1,2,3],"contractTitle":"Vestas 1","categoryTitle":"Ici'
            'ng","categoryTime":"available","value":104.55,"duration":150},{"aggregateId'
            '":6,"aggregatePathNames":[],"deviceIds":[1,2,3],"contractTitle":"Vestas 1",'
            '"categoryTitle":"Utility","categoryTime":"excluded","value":73,"duration":5'
            '0.3}]}'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Gets signal data aggregated per availability contract category.
    #
    #_🔐 This endpoint requires the **Data** and **Statuses** endpoint permissions._
    #
    #_This request can also be made using the POST method, 
    #with a request to `datapercategory.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_data_per_category_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        data_signal_id = 248
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        aggregate = 'device'
        aggregate_level = 0
        category = APIHelper.json_deserialize('["stop"]')

        # Perform the API call through the SDK function
        result = self.controller.get_data_per_category(device_ids, data_signal_id, timestamp_start, timestamp_end, aggregate, aggregate_level, category)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

class IntervalTypeEnum(object):

    """Implementation of the 'IntervalType' enum.

    The type of with which the task repeats.

    Attributes:
        DAY: TODO: type description here.
        WEEK: TODO: type description here.
        MONTH: TODO: type description here.
        YEAR: TODO: type description here.

    """

    DAY = 'day'

    WEEK = 'week'

    MONTH = 'month'

    YEAR = 'year'




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class PersonnelQualification(object):

    """Implementation of the 'PersonnelQualification' model.

    A personnel qualification.

    Attributes:
        qualification_id (int): The id of a personnel qualification.
        manufacturer (string): TODO: type description here.
        qualification_type (string): TODO: type description here.
        qualification_description (string): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "qualification_id":'qualificationId',
        "manufacturer":'manufacturer',
        "qualification_type":'qualificationType',
        "qualification_description":'qualificationDescription'
    }

    def __init__(self,
                 qualification_id=None,
                 manufacturer=None,
                 qualification_type=None,
                 qualification_description=None):
        """Constructor for the PersonnelQualification class"""

        # Initialize members of the class
        self.qualification_id = qualification_id
        self.manufacturer = manufacturer
        self.qualification_type = qualification_type
        self.qualification_description = qualification_description


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        qualification_id = dictionary.get('qualificationId')
        manufacturer = dictionary.get('manufacturer')
        qualification_type = dictionary.get('qualificationType')
        qualification_description = dictionary.get('qualificationDescription')

        # Return an object of this model
        return cls(qualification_id,
                   manufacturer,
                   qualification_type,
                   qualification_description)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper

class TimeZoneConfiguration(object):

    """Implementation of the 'TimeZoneConfiguration' model.

    The time zone configuration.

    Attributes:
        title (string): The title of the time zone.
        utc_offset (float): The UTC offset for the time zone.
        utc_offset_dst (float): The UTC offset for the time zone during
            daylight savings time.
        dst_timestamp_start (datetime): The start of daylight savings time in
            the time zone. This timestamp is given in UTC.
        dst_timestamp_end (datetime): The end of daylight savings time in the
            time zone. This timestamp is given in UTC.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "title":'title',
        "utc_offset":'utcOffset',
        "utc_offset_dst":'utcOffsetDst',
        "dst_timestamp_start":'dstTimestampStart',
        "dst_timestamp_end":'dstTimestampEnd'
    }

    def __init__(self,
                 title=None,
                 utc_offset=None,
                 utc_offset_dst=None,
                 dst_timestamp_start=None,
                 dst_timestamp_end=None):
        """Constructor for the TimeZoneConfiguration class"""

        # Initialize members of the class
        self.title = title
        self.utc_offset = utc_offset
        self.utc_offset_dst = utc_offset_dst
        self.dst_timestamp_start = APIHelper.RFC3339DateTime(dst_timestamp_start) if dst_timestamp_start else None
        self.dst_timestamp_end = APIHelper.RFC3339DateTime(dst_timestamp_end) if dst_timestamp_end else None


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        title = dictionary.get('title')
        utc_offset = dictionary.get('utcOffset')
        utc_offset_dst = dictionary.get('utcOffsetDst')
        dst_timestamp_start = APIHelper.RFC3339DateTime.from_value(dictionary.get("dstTimestampStart")).datetime if dictionary.get("dstTimestampStart") else None
        dst_timestamp_end = APIHelper.RFC3339DateTime.from_value(dictionary.get("dstTimestampEnd")).datetime if dictionary.get("dstTimestampEnd") else None

        # Return an object of this model
        return cls(title,
                   utc_offset,
                   utc_offset_dst,
                   dst_timestamp_start,
                   dst_timestamp_end)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
import greenbyteapi.models.user

class TasksFilesResponse(object):

    """Implementation of the 'Tasks Files Response' model.

    TODO: type model description here.

    Attributes:
        file_id (int): The id of a file.
        file_name (string): TODO: type description here.
        timestamp_uploaded (datetime): The timestamp when the file was
            uploaded.
        uploaded_by (User): TODO: type description here.
        description (string): TODO: type description here.
        category (CategoryEnum): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "file_id":'fileId',
        "file_name":'fileName',
        "timestamp_uploaded":'timestampUploaded',
        "uploaded_by":'uploadedBy',
        "description":'description',
        "category":'category'
    }

    def __init__(self,
                 file_id=None,
                 file_name=None,
                 timestamp_uploaded=None,
                 uploaded_by=None,
                 description=None,
                 category=None):
        """Constructor for the TasksFilesResponse class"""

        # Initialize members of the class
        self.file_id = file_id
        self.file_name = file_name
        self.timestamp_uploaded = APIHelper.RFC3339DateTime(timestamp_uploaded) if timestamp_uploaded else None
        self.uploaded_by = uploaded_by
        self.description = description
        self.category = category


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        file_id = dictionary.get('fileId')
        file_name = dictionary.get('fileName')
        timestamp_uploaded = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampUploaded")).datetime if dictionary.get("timestampUploaded") else None
        uploaded_by = greenbyteapi.models.user.User.from_dictionary(dictionary.get('uploadedBy')) if dictionary.get('uploadedBy') else None
        description = dictionary.get('description')
        category = dictionary.get('category')

        # Return an object of this model
        return cls(file_id,
                   file_name,
                   timestamp_uploaded,
                   uploaded_by,
                   description,
                   category)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper

class DeviceAccess(object):

    """Implementation of the 'DeviceAccess' model.

    A device access

    Attributes:
        device_access_id (int): The id of a device access.
        site_id (int): The id of a site.
        device_ids (list of int): The associated device ids.
        personnel_ids (list of int): The associated personnel ids.
        task_ids (list of int): The associated task ids.
        timestamp_start (datetime): The timestamp when the device access
            is/was planned to start. The timestamp is in the time zone
            configured in the Greenbyte Platform without UTC offset.
        timestamp_end_expected (datetime): The timestamp when the device
            access is/was planned to end. The timestamp is in the time zone
            configured in the Greenbyte Platform without UTC offset.
        timestamp_end (datetime): The timestamp when the device access
            actually ended. The timestamp is in the time zone configured in
            the Greenbyte Platform without UTC offset.
        log_on_comment (string): A comment for when logging on to the device.
        log_off_comment (string): A comment for when logging off from the
            device.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "device_access_id":'deviceAccessId',
        "site_id":'siteId',
        "device_ids":'deviceIds',
        "personnel_ids":'personnelIds',
        "task_ids":'taskIds',
        "timestamp_start":'timestampStart',
        "timestamp_end_expected":'timestampEndExpected',
        "timestamp_end":'timestampEnd',
        "log_on_comment":'logOnComment',
        "log_off_comment":'logOffComment'
    }

    def __init__(self,
                 device_access_id=None,
                 site_id=None,
                 device_ids=None,
                 personnel_ids=None,
                 task_ids=None,
                 timestamp_start=None,
                 timestamp_end_expected=None,
                 timestamp_end=None,
                 log_on_comment=None,
                 log_off_comment=None):
        """Constructor for the DeviceAccess class"""

        # Initialize members of the class
        self.device_access_id = device_access_id
        self.site_id = site_id
        self.device_ids = device_ids
        self.personnel_ids = personnel_ids
        self.task_ids = task_ids
        self.timestamp_start = APIHelper.RFC3339DateTime(timestamp_start) if timestamp_start else None
        self.timestamp_end_expected = APIHelper.RFC3339DateTime(timestamp_end_expected) if timestamp_end_expected else None
        self.timestamp_end = APIHelper.RFC3339DateTime(timestamp_end) if timestamp_end else None
        self.log_on_comment = log_on_comment
        self.log_off_comment = log_off_comment


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        device_access_id = dictionary.get('deviceAccessId')
        site_id = dictionary.get('siteId')
        device_ids = dictionary.get('deviceIds')
        personnel_ids = dictionary.get('personnelIds')
        task_ids = dictionary.get('taskIds')
        timestamp_start = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampStart")).datetime if dictionary.get("timestampStart") else None
        timestamp_end_expected = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampEndExpected")).datetime if dictionary.get("timestampEndExpected") else None
        timestamp_end = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampEnd")).datetime if dictionary.get("timestampEnd") else None
        log_on_comment = dictionary.get('logOnComment')
        log_off_comment = dictionary.get('logOffComment')

        # Return an object of this model
        return cls(device_access_id,
                   site_id,
                   device_ids,
                   personnel_ids,
                   task_ids,
                   timestamp_start,
                   timestamp_end_expected,
                   timestamp_end,
                   log_on_comment,
                   log_off_comment)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
import greenbyteapi.exceptions.api_exception

class ProblemDetailsException(greenbyteapi.exceptions.api_exception.APIException):
    def __init__(self, reason, context):
        """Constructor for the ProblemDetailsException class

        Args:
            reason (string): The reason (or error message) for the Exception
                to be raised.
            context (HttpContext):  The HttpContext of the API call.

        """
        super(ProblemDetailsException, self).__init__(reason, context)
        dictionary = APIHelper.json_deserialize(self.context.response.raw_body)
        if isinstance(dictionary, dict):
            self.unbox(dictionary)

    def unbox(self, dictionary):
        """Populates the properties of this object by extracting them from a dictionary.

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        """
        self.status = dictionary.get('status')
        self.title = dictionary.get('title')



# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper

class AlertItem(object):

    """Implementation of the 'AlertItem' model.

    An alert generated for a device based on a rule.

    Attributes:
        device_id (int): The id of the device the alert was generated for.
        rule_id (int): The id of the rule the alert is based on.
        timestamp_start (datetime): The timestamp when the alert began. The
            timestamp is in the time zone configured in the Greenbyte Platform
            without UTC offset.
        timestamp_end (datetime): The timestamp when the alert ended. The
            timestamp is in the time zone configured in the Greenbyte Platform
            without UTC offset.
        message (string): The title of the rule the alert is based on.
        comment (string): A user comment associated with the alert.
        description (string): A description explaning the rule the alert is
            based on.
        details (string): Additional details for the alert. Note that the
            structure of this data is subject to change.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "device_id":'deviceId',
        "rule_id":'ruleId',
        "timestamp_start":'timestampStart',
        "timestamp_end":'timestampEnd',
        "message":'message',
        "comment":'comment',
        "description":'description',
        "details":'details'
    }

    def __init__(self,
                 device_id=None,
                 rule_id=None,
                 timestamp_start=None,
                 timestamp_end=None,
                 message=None,
                 comment=None,
                 description=None,
                 details=None):
        """Constructor for the AlertItem class"""

        # Initialize members of the class
        self.device_id = device_id
        self.rule_id = rule_id
        self.timestamp_start = APIHelper.RFC3339DateTime(timestamp_start) if timestamp_start else None
        self.timestamp_end = APIHelper.RFC3339DateTime(timestamp_end) if timestamp_end else None
        self.message = message
        self.comment = comment
        self.description = description
        self.details = details


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        device_id = dictionary.get('deviceId')
        rule_id = dictionary.get('ruleId')
        timestamp_start = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampStart")).datetime if dictionary.get("timestampStart") else None
        timestamp_end = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampEnd")).datetime if dictionary.get("timestampEnd") else None
        message = dictionary.get('message')
        comment = dictionary.get('comment')
        description = dictionary.get('description')
        details = dictionary.get('details')

        # Return an object of this model
        return cls(device_id,
                   rule_id,
                   timestamp_start,
                   timestamp_end,
                   message,
                   comment,
                   description,
                   details)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class ClientConfiguration(object):

    """Implementation of the 'ClientConfiguration' model.

    General configuration data.

    Attributes:
        title (string): The title of your Greenbyte Platform website.
        tag (string): Your internal customer tag.
        url_web (string): Your URL to access the Greenbyte Platform website.
        url_api (string): Your URL to access the Greenbyte Platform API.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "title":'title',
        "tag":'tag',
        "url_web":'urlWeb',
        "url_api":'urlApi'
    }

    def __init__(self,
                 title=None,
                 tag=None,
                 url_web=None,
                 url_api=None):
        """Constructor for the ClientConfiguration class"""

        # Initialize members of the class
        self.title = title
        self.tag = tag
        self.url_web = url_web
        self.url_api = url_api


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        title = dictionary.get('title')
        tag = dictionary.get('tag')
        url_web = dictionary.get('urlWeb')
        url_api = dictionary.get('urlApi')

        # Return an object of this model
        return cls(title,
                   tag,
                   url_web,
                   url_api)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import greenbyteapi.models.organization
import greenbyteapi.models.personnel_qualification
import greenbyteapi.models.personnel_site_induction

class Personnel(object):

    """Implementation of the 'Personnel' model.

    A person in the personnel list.

    Attributes:
        personnel_id (int): The id of a person in the personnel list.
        first_name (string): TODO: type description here.
        last_name (string): TODO: type description here.
        email (string): TODO: type description here.
        phone (string): TODO: type description here.
        mobile (string): TODO: type description here.
        organization (Organization): An organization used for tasks and
            personnel.
        qualifications (list of PersonnelQualification): TODO: type
            description here.
        site_inductions (list of PersonnelSiteInduction): TODO: type
            description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "personnel_id":'personnelId',
        "first_name":'firstName',
        "last_name":'lastName',
        "email":'email',
        "phone":'phone',
        "mobile":'mobile',
        "organization":'organization',
        "qualifications":'qualifications',
        "site_inductions":'siteInductions'
    }

    def __init__(self,
                 personnel_id=None,
                 first_name=None,
                 last_name=None,
                 email=None,
                 phone=None,
                 mobile=None,
                 organization=None,
                 qualifications=None,
                 site_inductions=None):
        """Constructor for the Personnel class"""

        # Initialize members of the class
        self.personnel_id = personnel_id
        self.first_name = first_name
        self.last_name = last_name
        self.email = email
        self.phone = phone
        self.mobile = mobile
        self.organization = organization
        self.qualifications = qualifications
        self.site_inductions = site_inductions


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        personnel_id = dictionary.get('personnelId')
        first_name = dictionary.get('firstName')
        last_name = dictionary.get('lastName')
        email = dictionary.get('email')
        phone = dictionary.get('phone')
        mobile = dictionary.get('mobile')
        organization = greenbyteapi.models.organization.Organization.from_dictionary(dictionary.get('organization')) if dictionary.get('organization') else None
        qualifications = None
        if dictionary.get('qualifications') != None:
            qualifications = list()
            for structure in dictionary.get('qualifications'):
                qualifications.append(greenbyteapi.models.personnel_qualification.PersonnelQualification.from_dictionary(structure))
        site_inductions = None
        if dictionary.get('siteInductions') != None:
            site_inductions = list()
            for structure in dictionary.get('siteInductions'):
                site_inductions.append(greenbyteapi.models.personnel_site_induction.PersonnelSiteInduction.from_dictionary(structure))

        # Return an object of this model
        return cls(personnel_id,
                   first_name,
                   last_name,
                   email,
                   phone,
                   mobile,
                   organization,
                   qualifications,
                   site_inductions)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.decorators import lazy_property
from greenbyteapi.configuration import Configuration
from greenbyteapi.controllers.data_controller import DataController
from greenbyteapi.controllers.statuses_controller import StatusesController
from greenbyteapi.controllers.configuration_data_controller import ConfigurationDataController
from greenbyteapi.controllers.assets_controller import AssetsController
from greenbyteapi.controllers.alerts_controller import AlertsController
from greenbyteapi.controllers.plan_controller import PlanController


class GreenbyteapiClient(object):

    config = Configuration

    @lazy_property
    def data(self):
        return DataController()

    @lazy_property
    def statuses(self):
        return StatusesController()

    @lazy_property
    def configuration_data(self):
        return ConfigurationDataController()

    @lazy_property
    def assets(self):
        return AssetsController()

    @lazy_property
    def alerts(self):
        return AlertsController()

    @lazy_property
    def plan(self):
        return PlanController()


    def __init__(self,
                 x_api_key=None):
        if x_api_key is not None:
            Configuration.x_api_key = x_api_key




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

class ResolutionEnum(object):

    """Implementation of the 'Resolution' enum.

    The resolution for time-series data.

    Attributes:
        ENUM_5MINUTE: The data is presented in 5 minute resolution.
        ENUM_10MINUTE: The data is presented in 10 minute resolution.
        ENUM_15MINUTE: The data is presented in fifteen minute resolution.
        HOURLY: The data is presented in hourly resolution.
        DAILY: The data is presented in daily resolution.
        WEEKLY: The data is presented in weekly resolution.
        MONTHLY: The data is presented in monthly resolution.
        YEARLY: The data is presented in yearly resolution.
        INTERVAL: The data is presented as a single value for the interval.
        DEVICE: The device's own resolution is used.

    """

    ENUM_5MINUTE = '5minute'

    ENUM_10MINUTE = '10minute'

    ENUM_15MINUTE = '15minute'

    HOURLY = 'hourly'

    DAILY = 'daily'

    WEEKLY = 'weekly'

    MONTHLY = 'monthly'

    YEARLY = 'yearly'

    INTERVAL = 'interval'

    DEVICE = 'device'




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from greenbyteapi.api_helper import APIHelper


class ConfigurationDataControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(ConfigurationDataControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.configuration_data

    # Gets your system-wide configuration data.
    #
    #_🔐 This endpoint requires the **Configuration** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `configuration.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_configuration(self):

        # Perform the API call through the SDK function
        result = self.controller.get_configuration()

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"client":{"title":"Intro (Greenbyte AB)","tag":"intro","urlWeb":"https:/'
            '/intro.greenbyte.cloud/","urlApi":"https://intro.greenbyte.cloud/api/2.0/"}'
            ',"timeZone":{"title":"Europe/Stockholm","utcOffset":1,"utcOffsetDst":2,"dst'
            'TimestampStart":"2020-03-29T01:00:00","dstTimestampEnd":"2020-10-25T01:00:0'
            '0"},"dataSignals":{"availabilityTimeDataSignalId":430,"availabilityProducti'
            'onDataSignalId":445,"lostProductionDataSignalId":432,"performanceDataSignal'
            'Id":436}}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # Gets your system-wide configuration data.
    #
    #_🔐 This endpoint requires the **Configuration** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `configuration.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_configuration_1(self):

        # Perform the API call through the SDK function
        result = self.controller.get_configuration()

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class DataSignal(object):

    """Implementation of the 'DataSignal' model.

    A data signal.

    Attributes:
        data_signal_id (int): The unique id of a data signal.
        title (string): TODO: type description here.
        unit (string): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "data_signal_id":'dataSignalId',
        "title":'title',
        "unit":'unit'
    }

    def __init__(self,
                 data_signal_id=None,
                 title=None,
                 unit=None):
        """Constructor for the DataSignal class"""

        # Initialize members of the class
        self.data_signal_id = data_signal_id
        self.title = title
        self.unit = unit


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        data_signal_id = dictionary.get('dataSignalId')
        title = dictionary.get('title')
        unit = dictionary.get('unit')

        # Return an object of this model
        return cls(data_signal_id,
                   title,
                   unit)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from greenbyteapi.api_helper import APIHelper


class PlanControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(PlanControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.plan

    # **(BETA)** Gets a list of task categories.
    #
    #_🔐 This endpoint requires the **Plan** endpoint permission._
    #
    #_This is a beta feature. Some details might change before it is
    #released as a stable version._
    #
    def test_test_list_task_categories(self):

        # Perform the API call through the SDK function
        result = self.controller.list_task_categories()

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"categoryId":10,"title":"Scheduled maintenance"},{"categoryId":20,"title'
            '":"Unscheduled maintenance"}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # **(BETA)** Gets a list of downtime events.
    #
    #_🔐 This endpoint requires the **Plan** endpoint permission._
    #
    #_This is a beta feature. Some details might change before it is
    #released as a stable version._
    #
    def test_test_list_downtime_events(self):
        # Parameters for the API call
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        site_ids = APIHelper.json_deserialize('[1,2,3]')
        fields = APIHelper.json_deserialize('["deviceIds","timestampStart"]')
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.list_downtime_events(timestamp_start, timestamp_end, device_ids, site_ids, fields, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"downtimeEventId":1,"deviceIds":[1,2,3],"timestampStart":"2020-01-01T00:'
            '00:00","timestampEnd":"2020-01-02T00:00:00","comment":"Planned downtime","s'
            'iteIds":[],"taskIds":[1358]},{"downtimeEventId":2,"deviceIds":[1,2,3],"time'
            'stampStart":"2020-01-10T00:00:00","timestampEnd":"2020-01-12T00:00:00","com'
            'ment":"Unplanned downtime","siteIds":[1],"taskIds":[1359]}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # **(BETA)** Gets a list of site accesses.
    #
    #_🔐 This endpoint requires the **Plan** endpoint permission._
    #
    #_This is a beta feature. Some details might change before it is
    #released as a stable version._
    #
    def test_test_list_site_accesses(self):
        # Parameters for the API call
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        site_ids = APIHelper.json_deserialize('[1,2,3]')
        fields = APIHelper.json_deserialize('["siteAccessId","timestampStart"]')
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.list_site_accesses(timestamp_start, timestamp_end, device_ids, site_ids, fields, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"siteAccessId":10,"siteId":18,"deviceIds":[11,12],"taskIds":[66],"siteAc'
            'cessPersonnel":[{"personnelId":1234,"firstName":"Andreas","lastName":"Jonss'
            'on","company":"Power Offtakers Inc.","phoneNumber":"456-123","vehicleRegist'
            'ration":"ABC456","comment":"Site access comment","timestampStart":"2020-04-'
            '30T10:03:00","timestampEnd":"2020-04-30T17:39:00"}],"timestampStart":"2020-'
            '01-01T12:00:00","timestampEndExpected":"2020-01-01T13:00:00","timestampEnd"'
            ':"2020-01-01T13:30:00","logOnComment":"Investigating","logOffComment":"All '
            'clear"},{"siteAccessId":11,"siteId":18,"deviceIds":[15],"taskIds":[55,56],"'
            'siteAccessPersonnel":[{"personnelId":2345,"firstName":"Andrea","lastName":"'
            'Larsson","company":"Power Offtakers Inc.","phoneNumber":"123-456","vehicleR'
            'egistration":"ABC123","comment":"Site access comment","timestampStart":"202'
            '0-04-30T10:04:00","timestampEnd":"2020-04-30T17:39:00"}],"timestampStart":"'
            '2020-01-02T12:00:00","timestampEndExpected":"2020-01-02T13:00:00","timestam'
            'pEnd":"2020-01-02T13:30:00","logOnComment":"Investigating","logOffComment":'
            '"All clear"}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # **(BETA)** Gets a list of organizations.
    #
    #_🔐 This endpoint requires the **Plan** endpoint permission._
    #
    #_This is a beta feature. Some details might change before it is
    #released as a stable version._
    #
    def test_test_list_organizations(self):

        # Perform the API call through the SDK function
        result = self.controller.list_organizations()

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"organizationId":10,"name":"Power Offtakers Inc.","email":"support@power'
            '-offtakers.example.com","phone":"456-789"}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))


    # **(BETA)** Gets a list of personnel.
    #
    #_🔐 This endpoint requires the **Plan** endpoint permission._
    #
    #_This is a beta feature. Some details might change before it is
    #released as a stable version._
    #
    def test_test_list_personnel(self):
        # Parameters for the API call
        fields = APIHelper.json_deserialize('["lastName","phone"]')
        page_size = 50
        page = 1

        # Perform the API call through the SDK function
        result = self.controller.list_personnel(fields, page_size, page)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))

        
        # Test whether the captured response is as we expected
        self.assertIsNotNone(result)
        expected_body = APIHelper.json_deserialize(
            '[{"personnelId":5,"firstName":"Greenbyte","lastName":"Support","email":"su'
            'pport@greenbyte.com","phone":"123-456","mobile":"654-321","organization":{"'
            'organizationId":10,"name":"Power Offtakers Inc.","email":"support@power-off'
            'takers.example.com","phone":"456-789"},"qualifications":[{"qualificationId"'
            ':85,"manufacturer":"GE","qualificationType":"AP","qualificationDescription"'
            ':"Authorized Person"}],"siteInductions":[{"siteInductionId":43,"siteId":1,"'
            'dateExpires":"2020-12-01"}]}]'
            )
        received_body = APIHelper.json_deserialize(self.response_catcher.response.raw_body)
        self.assertTrue(TestHelper.match_body(expected_body, received_body))





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class DataPerCategoryItem(object):

    """Implementation of the 'DataPerCategoryItem' model.

    Data for a single aggregate group and contract category combination.

    Attributes:
        aggregate_id (int): The id of this aggregate group: device id, site
            id, or the constant -1 for portfolio. For `siteLevel` aggregation
            a generated unique id is used.
        aggregate_path_names (list of string): For `siteLevel` aggregation
            this contains the title for each level in the hierarchy. For other
            types of aggregation it will be empty.
        device_ids (list of int): The ids of the devices in this aggregate
            group.
        contract_title (string): TODO: type description here.
        category_title (string): TODO: type description here.
        category_time (CategoryTimeEnum): TODO: type description here.
        value (float): The aggregated value of the selected data signal.
        duration (float): The summed duration in seconds allocated to this
            contract category and aggregate group.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "aggregate_id":'aggregateId',
        "device_ids":'deviceIds',
        "contract_title":'contractTitle',
        "category_title":'categoryTitle',
        "category_time":'categoryTime',
        "value":'value',
        "aggregate_path_names":'aggregatePathNames',
        "duration":'duration'
    }

    def __init__(self,
                 aggregate_id=None,
                 device_ids=None,
                 contract_title=None,
                 category_title=None,
                 category_time=None,
                 value=None,
                 aggregate_path_names=None,
                 duration=None):
        """Constructor for the DataPerCategoryItem class"""

        # Initialize members of the class
        self.aggregate_id = aggregate_id
        self.aggregate_path_names = aggregate_path_names
        self.device_ids = device_ids
        self.contract_title = contract_title
        self.category_title = category_title
        self.category_time = category_time
        self.value = value
        self.duration = duration


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        aggregate_id = dictionary.get('aggregateId')
        device_ids = dictionary.get('deviceIds')
        contract_title = dictionary.get('contractTitle')
        category_title = dictionary.get('categoryTitle')
        category_time = dictionary.get('categoryTime')
        value = dictionary.get('value')
        aggregate_path_names = dictionary.get('aggregatePathNames')
        duration = dictionary.get('duration')

        # Return an object of this model
        return cls(aggregate_id,
                   device_ids,
                   contract_title,
                   category_title,
                   category_time,
                   value,
                   aggregate_path_names,
                   duration)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
import greenbyteapi.models.site
import greenbyteapi.models.device_model
import greenbyteapi.models.turbine_type
import greenbyteapi.models.metadata_field

class Device(object):

    """Implementation of the 'Device' model.

    TODO: type model description here.

    Attributes:
        device_id (int): The id of a device.
        title (string): TODO: type description here.
        alt_title (string): An alternative title.
        identity (string): Device identification number.
        site (Site): The site where the device is located.
        device_type (string): The string representation of the device type.
        device_type_id (int): The id of a device type.
        parent_id (int): The id of the parent device, if any.
        child_ids (list of int): Ids of child devices, if any.
        device_model (DeviceModel): Information about the device manufacturer
            and model. Applies to devices of device type **other than**
            `Turbine`.
        turbine_type (TurbineType): Turbine type information (manufacturer and
            model). Only applies to devices of device type `Turbine`.
        max_power (int): The maximum power for a device.
        bidding_area (string): Only applies to Nordic countries and the UK.
        timestamp_start (datetime): The earliest timestamp device data is
            available for.
        latitude (string): The latitude of the device in the WGS84 system.
        longitude (string): The longitude of the device in the WGS84 system.
        elevation (string): The elevation of the device in meters above sea
            level.
        target_availability (float): The target availability for the device.
        metadata (list of MetadataField): A list of metadata fields and their
            values.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "device_id":'deviceId',
        "title":'title',
        "alt_title":'altTitle',
        "identity":'identity',
        "site":'site',
        "device_type":'deviceType',
        "device_type_id":'deviceTypeId',
        "parent_id":'parentId',
        "child_ids":'childIds',
        "device_model":'deviceModel',
        "turbine_type":'turbineType',
        "max_power":'maxPower',
        "bidding_area":'biddingArea',
        "timestamp_start":'timestampStart',
        "latitude":'latitude',
        "longitude":'longitude',
        "elevation":'elevation',
        "target_availability":'targetAvailability',
        "metadata":'metadata'
    }

    def __init__(self,
                 device_id=None,
                 title=None,
                 alt_title=None,
                 identity=None,
                 site=None,
                 device_type=None,
                 device_type_id=None,
                 parent_id=None,
                 child_ids=None,
                 device_model=None,
                 turbine_type=None,
                 max_power=None,
                 bidding_area=None,
                 timestamp_start=None,
                 latitude=None,
                 longitude=None,
                 elevation=None,
                 target_availability=None,
                 metadata=None):
        """Constructor for the Device class"""

        # Initialize members of the class
        self.device_id = device_id
        self.title = title
        self.alt_title = alt_title
        self.identity = identity
        self.site = site
        self.device_type = device_type
        self.device_type_id = device_type_id
        self.parent_id = parent_id
        self.child_ids = child_ids
        self.device_model = device_model
        self.turbine_type = turbine_type
        self.max_power = max_power
        self.bidding_area = bidding_area
        self.timestamp_start = APIHelper.RFC3339DateTime(timestamp_start) if timestamp_start else None
        self.latitude = latitude
        self.longitude = longitude
        self.elevation = elevation
        self.target_availability = target_availability
        self.metadata = metadata


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        device_id = dictionary.get('deviceId')
        title = dictionary.get('title')
        alt_title = dictionary.get('altTitle')
        identity = dictionary.get('identity')
        site = greenbyteapi.models.site.Site.from_dictionary(dictionary.get('site')) if dictionary.get('site') else None
        device_type = dictionary.get('deviceType')
        device_type_id = dictionary.get('deviceTypeId')
        parent_id = dictionary.get('parentId')
        child_ids = dictionary.get('childIds')
        device_model = greenbyteapi.models.device_model.DeviceModel.from_dictionary(dictionary.get('deviceModel')) if dictionary.get('deviceModel') else None
        turbine_type = greenbyteapi.models.turbine_type.TurbineType.from_dictionary(dictionary.get('turbineType')) if dictionary.get('turbineType') else None
        max_power = dictionary.get('maxPower')
        bidding_area = dictionary.get('biddingArea')
        timestamp_start = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampStart")).datetime if dictionary.get("timestampStart") else None
        latitude = dictionary.get('latitude')
        longitude = dictionary.get('longitude')
        elevation = dictionary.get('elevation')
        target_availability = dictionary.get('targetAvailability')
        metadata = None
        if dictionary.get('metadata') != None:
            metadata = list()
            for structure in dictionary.get('metadata'):
                metadata.append(greenbyteapi.models.metadata_field.MetadataField.from_dictionary(structure))

        # Return an object of this model
        return cls(device_id,
                   title,
                   alt_title,
                   identity,
                   site,
                   device_type,
                   device_type_id,
                   parent_id,
                   child_ids,
                   device_model,
                   turbine_type,
                   max_power,
                   bidding_area,
                   timestamp_start,
                   latitude,
                   longitude,
                   elevation,
                   target_availability,
                   metadata)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

class AggregateModeEnum(object):

    """Implementation of the 'AggregateMode' enum.

    How data is aggregated in the asset structure.

    Attributes:
        DEVICE: The data is aggregated by device.
        DEVICELEVEL: The data is aggregated by top-level device.
        SITE: The data is aggregated by site.
        PORTFOLIO: All data is aggregated into a single group.
        SITELEVEL: The data is aggregated by site hierarchy level. When this
            mode is used the parameter AggregateLevel controls down to which
            level in the hierarchy to aggregate.

    """

    DEVICE = 'device'

    DEVICELEVEL = 'deviceLevel'

    SITE = 'site'

    PORTFOLIO = 'portfolio'

    SITELEVEL = 'siteLevel'




# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class User(object):

    """Implementation of the 'User' model.

    TODO: type model description here.

    Attributes:
        first_name (string): TODO: type description here.
        last_name (string): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "first_name":'firstName',
        "last_name":'lastName'
    }

    def __init__(self,
                 first_name=None,
                 last_name=None):
        """Constructor for the User class"""

        # Initialize members of the class
        self.first_name = first_name
        self.last_name = last_name


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        first_name = dictionary.get('firstName')
        last_name = dictionary.get('lastName')

        # Return an object of this model
        return cls(first_name,
                   last_name)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import greenbyteapi.models.power_curve_value

class PowerCurve(object):

    """Implementation of the 'PowerCurve' model.

    TODO: type model description here.

    Attributes:
        device_id (int): The id of a device.
        title (string): The title of the power curve.
        values (list of PowerCurveValue): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "device_id":'deviceId',
        "title":'title',
        "values":'values'
    }

    def __init__(self,
                 device_id=None,
                 title=None,
                 values=None):
        """Constructor for the PowerCurve class"""

        # Initialize members of the class
        self.device_id = device_id
        self.title = title
        self.values = values


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        device_id = dictionary.get('deviceId')
        title = dictionary.get('title')
        values = None
        if dictionary.get('values') != None:
            values = list()
            for structure in dictionary.get('values'):
                values.append(greenbyteapi.models.power_curve_value.PowerCurveValue.from_dictionary(structure))

        # Return an object of this model
        return cls(device_id,
                   title,
                   values)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class TurbineType(object):

    """Implementation of the 'TurbineType' model.

    Turbine-specific type information.

    Attributes:
        turbine_type_id (int): TODO: type description here.
        title (string): TODO: type description here.
        manufacturer (string): TODO: type description here.
        model (string): TODO: type description here.
        controller (string): The model of the turbine controller.
        rated_power (int): TODO: type description here.
        max_rotor_speed (float): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "turbine_type_id":'turbineTypeId',
        "title":'title',
        "manufacturer":'manufacturer',
        "model":'model',
        "controller":'controller',
        "rated_power":'ratedPower',
        "max_rotor_speed":'maxRotorSpeed'
    }

    def __init__(self,
                 turbine_type_id=None,
                 title=None,
                 manufacturer=None,
                 model=None,
                 controller=None,
                 rated_power=None,
                 max_rotor_speed=None):
        """Constructor for the TurbineType class"""

        # Initialize members of the class
        self.turbine_type_id = turbine_type_id
        self.title = title
        self.manufacturer = manufacturer
        self.model = model
        self.controller = controller
        self.rated_power = rated_power
        self.max_rotor_speed = max_rotor_speed


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        turbine_type_id = dictionary.get('turbineTypeId')
        title = dictionary.get('title')
        manufacturer = dictionary.get('manufacturer')
        model = dictionary.get('model')
        controller = dictionary.get('controller')
        rated_power = dictionary.get('ratedPower')
        max_rotor_speed = dictionary.get('maxRotorSpeed')

        # Return an object of this model
        return cls(turbine_type_id,
                   title,
                   manufacturer,
                   model,
                   controller,
                   rated_power,
                   max_rotor_speed)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import greenbyteapi.models.data_signal
import greenbyteapi.models.data_per_category_item

class DataPerCategoryResponse(object):

    """Implementation of the 'DataPerCategoryResponse' model.

    An object containing data grouped by contract category and aggregate.

    Attributes:
        data_signal (DataSignal): A data signal.
        calculation (CalculationModeEnum): Which operation to use when
            aggregating data.
        data (list of DataPerCategoryItem): A list of objects: one per
            combination of * aggregate * contract category

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "data_signal":'dataSignal',
        "calculation":'calculation',
        "data":'data'
    }

    def __init__(self,
                 data_signal=None,
                 calculation=None,
                 data=None):
        """Constructor for the DataPerCategoryResponse class"""

        # Initialize members of the class
        self.data_signal = data_signal
        self.calculation = calculation
        self.data = data


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        data_signal = greenbyteapi.models.data_signal.DataSignal.from_dictionary(dictionary.get('dataSignal')) if dictionary.get('dataSignal') else None
        calculation = dictionary.get('calculation')
        data = None
        if dictionary.get('data') != None:
            data = list()
            for structure in dictionary.get('data'):
                data.append(greenbyteapi.models.data_per_category_item.DataPerCategoryItem.from_dictionary(structure))

        # Return an object of this model
        return cls(data_signal,
                   calculation,
                   data)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
from greenbyteapi.configuration import Configuration
from greenbyteapi.controllers.base_controller import BaseController
from greenbyteapi.http.auth.custom_header_auth import CustomHeaderAuth
from greenbyteapi.models.status_item import StatusItem
from greenbyteapi.exceptions.problem_details_exception import ProblemDetailsException
from greenbyteapi.exceptions.api_exception import APIException

class StatusesController(BaseController):

    """A Controller to access Endpoints in the greenbyteapi API."""


    def get_statuses(self,
                     device_ids,
                     timestamp_start,
                     timestamp_end,
                     category=None,
                     lost_production_signal_id=None,
                     fields=None,
                     sort_by=None,
                     sort_asc=False,
                     page_size=50,
                     page=1,
                     use_utc=False):
        """Does a GET request to /status.

        Gets statuses for multiple devices during the given time period.
        The timestamps are in the time zone configured in the Greenbyte
        Platform.
        Use the useUtc flag to get timestamps in UTC.
        _🔐 This endpoint requires the **Statuses** endpoint permission._
        _This request can also be made using the POST method, 
        with a request to `status.json` and 
        a JSON request body instead of query parameters._

        Args:
            device_ids (list of int): Which devices to get statuses for.
            timestamp_start (datetime): The beginning of the time interval to
                get data for (inclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                start timestamp **is** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            timestamp_end (datetime): The end of the time interval to get data
                for (exclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                end timestamp is **not** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            category (list of StatusCategoryEnum, optional): Which status
                categories to get statuses for.
            lost_production_signal_id (int, optional): Which data signal to
                use for calculating lost production. Defaults to the
                configured default lost production signal.
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the `StatusItem`
                schema (See Response Type). By default all fields are
                included.
            sort_by (list of string, optional): Which fields to sort the
                response items by.
            sort_asc (bool, optional): Whether to sort the items in ascending
                order.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            list of StatusItem: Response from the API. A list of statuses with
                related data.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/status'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'deviceIds': device_ids,
            'timestampStart': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_start),
            'timestampEnd': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_end),
            'category': category,
            'lostProductionSignalId': lost_production_signal_id,
            'fields': fields,
            'sortBy': sort_by,
            'sortAsc': sort_asc,
            'pageSize': page_size,
            'page': page,
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, StatusItem.from_dictionary)

    def get_active_statuses(self,
                            device_ids,
                            category=None,
                            lost_production_signal_id=None,
                            fields=None,
                            sort_by=None,
                            sort_asc=False,
                            page_size=50,
                            page=1,
                            use_utc=False):
        """Does a GET request to /activestatus.

        Gets active statuses for multiple devices.
        The timestamps are in the time zone configured in the Greenbyte
        Platform.
        Use the useUtc flag to get timestamps in UTC.
        _🔐 This endpoint requires the **Statuses** endpoint permission._
        _This request can also be made using the POST method, 
        with a request to `activestatus.json` and 
        a JSON request body instead of query parameters._

        Args:
            device_ids (list of int): Which devices to get statuses for.
            category (list of StatusCategoryEnum, optional): Which status
                categories to get statuses for.
            lost_production_signal_id (int, optional): Which data signal to
                use for calculating lost production. Defaults to the
                configured default lost production signal.
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the `StatusItem`
                schema (See Response Type). By default all fields are
                included.
            sort_by (list of string, optional): Which fields to sort the
                response items by.
            sort_asc (bool, optional): Whether to sort the items in ascending
                order.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            list of StatusItem: Response from the API. A list of active
                statuses with related data.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/activestatus'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'deviceIds': device_ids,
            'category': category,
            'lostProductionSignalId': lost_production_signal_id,
            'fields': fields,
            'sortBy': sort_by,
            'sortAsc': sort_asc,
            'pageSize': page_size,
            'page': page,
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, StatusItem.from_dictionary)



# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
import greenbyteapi.models.data_signal
import greenbyteapi.models.status_item

class StatusItem(object):

    """Implementation of the 'StatusItem' model.

    A status that may contain statuses of the same type as sub-statuses. Note
    that for sub-statuses the fields `categoryIec`, `categoryContract`, and
    `subStatus` will always be null.

    Attributes:
        turbine_status_id (int): The id of a turbine status.
        device_id (int): The id of the device affected by the status.
        timestamp_start (datetime): The timestamp when the status began. The
            timestamp is in the time zone configured in the Greenbyte Platform
            without UTC offset.
        timestamp_end (datetime): The timestamp when the status ended. The
            timestamp is in the time zone configured in the Greenbyte Platform
            without UTC offset.
        has_timestamp_end (bool): Indicates whether the status has a
            duration.
        category (StatusCategoryEnum): The category a status belongs to.
        code (float): The status code.
        message (string): A description of the status code.
        comment (string): A user comment associated with the status.
        lost_production_signal (DataSignal): A data signal.
        lost_production (float): The lost production in kWh associated with
            the status. This field will be null if the caller is not
            authorized for the system-configured lost production signal. The
            configured lost production signal is available via the
            `/configuration.json` endpoint (`DataSignalConfiguration`
            schema).
        category_iec (string): The status category as defined by the IEC.
        category_contract (string): The status category as defined in the
            availability contract assigned to the site.
        category_contract_global (string): The status category as defined in
            the global availability contract assigned to the site.
        category_contract_custom (string): The status category as defined in
            the custom availability contract assigned to the site.
        sub_status (list of StatusItem): Statuses of the same type that have
            been grouped under this status.
        acknowledged (bool): Indicates whether the status has been
            acknowledged.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "turbine_status_id":'turbineStatusId',
        "device_id":'deviceId',
        "timestamp_start":'timestampStart',
        "timestamp_end":'timestampEnd',
        "has_timestamp_end":'hasTimestampEnd',
        "category":'category',
        "code":'code',
        "message":'message',
        "comment":'comment',
        "lost_production_signal":'lostProductionSignal',
        "lost_production":'lostProduction',
        "category_iec":'categoryIec',
        "category_contract":'categoryContract',
        "category_contract_global":'categoryContractGlobal',
        "category_contract_custom":'categoryContractCustom',
        "sub_status":'subStatus',
        "acknowledged":'acknowledged'
    }

    def __init__(self,
                 turbine_status_id=None,
                 device_id=None,
                 timestamp_start=None,
                 timestamp_end=None,
                 has_timestamp_end=None,
                 category=None,
                 code=None,
                 message=None,
                 comment=None,
                 lost_production_signal=None,
                 lost_production=None,
                 category_iec=None,
                 category_contract=None,
                 category_contract_global=None,
                 category_contract_custom=None,
                 sub_status=None,
                 acknowledged=None):
        """Constructor for the StatusItem class"""

        # Initialize members of the class
        self.turbine_status_id = turbine_status_id
        self.device_id = device_id
        self.timestamp_start = APIHelper.RFC3339DateTime(timestamp_start) if timestamp_start else None
        self.timestamp_end = APIHelper.RFC3339DateTime(timestamp_end) if timestamp_end else None
        self.has_timestamp_end = has_timestamp_end
        self.category = category
        self.code = code
        self.message = message
        self.comment = comment
        self.lost_production_signal = lost_production_signal
        self.lost_production = lost_production
        self.category_iec = category_iec
        self.category_contract = category_contract
        self.category_contract_global = category_contract_global
        self.category_contract_custom = category_contract_custom
        self.sub_status = sub_status
        self.acknowledged = acknowledged


    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        turbine_status_id = dictionary.get('turbineStatusId')
        device_id = dictionary.get('deviceId')
        timestamp_start = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampStart")).datetime if dictionary.get("timestampStart") else None
        timestamp_end = APIHelper.RFC3339DateTime.from_value(dictionary.get("timestampEnd")).datetime if dictionary.get("timestampEnd") else None
        has_timestamp_end = dictionary.get('hasTimestampEnd')
        category = dictionary.get('category')
        code = dictionary.get('code')
        message = dictionary.get('message')
        comment = dictionary.get('comment')
        lost_production_signal = greenbyteapi.models.data_signal.DataSignal.from_dictionary(dictionary.get('lostProductionSignal')) if dictionary.get('lostProductionSignal') else None
        lost_production = dictionary.get('lostProduction')
        category_iec = dictionary.get('categoryIec')
        category_contract = dictionary.get('categoryContract')
        category_contract_global = dictionary.get('categoryContractGlobal')
        category_contract_custom = dictionary.get('categoryContractCustom')
        sub_status = None
        if dictionary.get('subStatus') != None:
            sub_status = list()
            for structure in dictionary.get('subStatus'):
                sub_status.append(greenbyteapi.models.status_item.StatusItem.from_dictionary(structure))
        acknowledged = dictionary.get('acknowledged')

        # Return an object of this model
        return cls(turbine_status_id,
                   device_id,
                   timestamp_start,
                   timestamp_end,
                   has_timestamp_end,
                   category,
                   code,
                   message,
                   comment,
                   lost_production_signal,
                   lost_production,
                   category_iec,
                   category_contract,
                   category_contract_global,
                   category_contract_custom,
                   sub_status,
                   acknowledged)





# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
from greenbyteapi.configuration import Configuration
from greenbyteapi.controllers.base_controller import BaseController
from greenbyteapi.http.auth.custom_header_auth import CustomHeaderAuth
from greenbyteapi.models.configuration_item import ConfigurationItem
from greenbyteapi.exceptions.problem_details_exception import ProblemDetailsException
from greenbyteapi.exceptions.api_exception import APIException

class ConfigurationDataController(BaseController):

    """A Controller to access Endpoints in the greenbyteapi API."""


    def get_configuration(self):
        """Does a GET request to /configuration.

        Gets your system-wide configuration data.
        _🔐 This endpoint requires the **Configuration** endpoint
        permission._
        _This request can also be made using the POST method, 
        with a request to `configuration.json` and 
        a JSON request body instead of query parameters._

        Returns:
            list of ConfigurationItem: Response from the API. An array with a
                single item containing configuration data.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/configuration'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, ConfigurationItem.from_dictionary)



# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from greenbyteapi.api_helper import APIHelper
from greenbyteapi.configuration import Configuration
from greenbyteapi.controllers.base_controller import BaseController
from greenbyteapi.http.auth.custom_header_auth import CustomHeaderAuth
from greenbyteapi.models.task import Task
from greenbyteapi.models.task_category import TaskCategory
from greenbyteapi.models.task_comment import TaskComment
from greenbyteapi.models.tasks_files_response import TasksFilesResponse
from greenbyteapi.models.downtime_event import DowntimeEvent
from greenbyteapi.models.site_access import SiteAccess
from greenbyteapi.models.device_access import DeviceAccess
from greenbyteapi.models.organization import Organization
from greenbyteapi.models.personnel import Personnel
from greenbyteapi.exceptions.problem_details_exception import ProblemDetailsException
from greenbyteapi.exceptions.api_exception import APIException

class PlanController(BaseController):

    """A Controller to access Endpoints in the greenbyteapi API."""


    def list_tasks(self,
                   timestamp_start,
                   timestamp_end,
                   device_ids=None,
                   site_ids=None,
                   category_ids=None,
                   state=None,
                   fields=None,
                   page_size=50,
                   page=1,
                   use_utc=False):
        """Does a GET request to /tasks.

        **(BETA)** Gets a list of tasks.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            timestamp_start (datetime): The beginning of the time interval to
                get data for (inclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                start timestamp **is** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            timestamp_end (datetime): The end of the time interval to get data
                for (exclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                end timestamp is **not** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            device_ids (list of int, optional): What devices to get tasks
                for.
            site_ids (list of int, optional): What sites to get tasks for.
            category_ids (list of int, optional): What task categories to
                include.
            state (TaskStateEnum, optional): What state of tasks to get:
                resolved and unresolved. If not set, both resolved and
                unresolved tasks are included.
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the `Task` schema
                (See Response Type). By default all fields are included.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            list of Task: Response from the API. A list of tasks matching the
                filter parameters.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/tasks'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'timestampStart': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_start),
            'timestampEnd': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_end),
            'deviceIds': device_ids,
            'siteIds': site_ids,
            'categoryIds': category_ids,
            'state': state,
            'fields': fields,
            'pageSize': page_size,
            'page': page,
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, Task.from_dictionary)

    def get_task(self,
                 task_id,
                 use_utc=False):
        """Does a GET request to /tasks/{taskId}.

        **(BETA)** Get a single task by ID.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            task_id (int): The id of the task to get.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            Task: Response from the API. A single task.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/tasks/{taskId}'
        _url_path = APIHelper.append_url_with_template_parameters(_url_path, { 
            'taskId': task_id
        })
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 404:
            raise APIException('The requested resource could not be found.', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, Task.from_dictionary)

    def list_task_categories(self):
        """Does a GET request to /task-categories.

        **(BETA)** Gets a list of task categories.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Returns:
            list of TaskCategory: Response from the API. A list of task
                categories.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/task-categories'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, TaskCategory.from_dictionary)

    def list_task_comments(self,
                           task_id,
                           fields=None,
                           page_size=50,
                           page=1,
                           use_utc=False):
        """Does a GET request to /tasks/{taskId}/comments.

        **(BETA)** Gets a list of comments belonging to a task.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            task_id (int): The id of the task.
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the `TaskComment`
                schema (See Response Type). By default all fields are
                included.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            list of TaskComment: Response from the API. A list of comments
                belonging to the task.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/tasks/{taskId}/comments'
        _url_path = APIHelper.append_url_with_template_parameters(_url_path, { 
            'taskId': task_id
        })
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'fields': fields,
            'pageSize': page_size,
            'page': page,
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, TaskComment.from_dictionary)

    def list_task_files(self,
                        task_id,
                        fields=None,
                        page_size=50,
                        page=1,
                        use_utc=False):
        """Does a GET request to /tasks/{taskId}/files.

        **(BETA)** Gets a list of files belonging to a task.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            task_id (int): The id of the task.
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the `TaskFile`
                schema (See Response Type). By default all fields are
                included.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            list of TasksFilesResponse: Response from the API. A list with
                information about files belonging to the task.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/tasks/{taskId}/files'
        _url_path = APIHelper.append_url_with_template_parameters(_url_path, { 
            'taskId': task_id
        })
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'fields': fields,
            'pageSize': page_size,
            'page': page,
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 404:
            raise APIException('The requested resource could not be found.', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, TasksFilesResponse.from_dictionary)

    def download_task_file(self,
                           task_id,
                           file_id):
        """Does a GET request to /tasks/{taskId}/files/{fileId}/content.

        **(BETA)** Downloads a file belonging to a task.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            task_id (int): The id of the task.
            file_id (int): The id of the file.

        Returns:
            binary: Response from the API. The contents of a file linked to
                the task.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/tasks/{taskId}/files/{fileId}/content'
        _url_path = APIHelper.append_url_with_template_parameters(_url_path, { 
            'taskId': task_id,
            'fileId': file_id
        })
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare and execute request
        _request = self.http_client.get(_query_url)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request, binary = True)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 404:
            raise APIException('The requested resource could not be found.', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return _context.response.raw_body

    def list_downtime_events(self,
                             timestamp_start,
                             timestamp_end,
                             device_ids=None,
                             site_ids=None,
                             fields=None,
                             page_size=50,
                             page=1,
                             use_utc=False):
        """Does a GET request to /downtime-events.

        **(BETA)** Gets a list of downtime events.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            timestamp_start (datetime): The beginning of the time interval to
                get data for (inclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                start timestamp **is** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            timestamp_end (datetime): The end of the time interval to get data
                for (exclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                end timestamp is **not** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            device_ids (list of int, optional): What devices to get downtime
                events for.
            site_ids (list of int, optional): What sites to get downtime
                events for.
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the
                `DowntimeEvent` schema (See Response Type). By default all
                fields are included.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            list of DowntimeEvent: Response from the API. A list of downtime
                events matching the filter parameters.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/downtime-events'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'timestampStart': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_start),
            'timestampEnd': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_end),
            'deviceIds': device_ids,
            'siteIds': site_ids,
            'fields': fields,
            'pageSize': page_size,
            'page': page,
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, DowntimeEvent.from_dictionary)

    def get_downtime_event(self,
                           downtime_event_id,
                           use_utc=False):
        """Does a GET request to /downtime-events/{downtimeEventId}.

        **(BETA)** Gets a single downtime event by ID.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            downtime_event_id (int): The id of the downtime event to get.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            DowntimeEvent: Response from the API. A single downtime event.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/downtime-events/{downtimeEventId}'
        _url_path = APIHelper.append_url_with_template_parameters(_url_path, { 
            'downtimeEventId': downtime_event_id
        })
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 404:
            raise APIException('The requested resource could not be found.', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, DowntimeEvent.from_dictionary)

    def list_site_accesses(self,
                           timestamp_start,
                           timestamp_end,
                           device_ids=None,
                           site_ids=None,
                           fields=None,
                           page_size=50,
                           page=1,
                           use_utc=False):
        """Does a GET request to /site-accesses.

        **(BETA)** Gets a list of site accesses.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            timestamp_start (datetime): The beginning of the time interval to
                get data for (inclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                start timestamp **is** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            timestamp_end (datetime): The end of the time interval to get data
                for (exclusive), in [RFC 3339, section
                5.6](https://tools.ietf.org/html/rfc3339#section-5.6)
                **date-time** format:  * Timestamps ending with 'Z' are
                treated as UTC. Example: "2020-01-01T00:00:00Z" * Time zone
                (UTC) offset timestamps ending with '+HH:mm'/"-HH:mm" are also
                supported. Example: "2020-01-01T02:00:00-02:00" * Other
                timestamps are treated as being in the time zone configured in
                the Greenbyte Platform. Example: "2020-01-01T00:00:00"  The
                end timestamp is **not** included in the time interval: for
                example, to select the full month of March 2020, set
                `timestampStart` to "2020-03-01T00:00:00" and `timestampEnd`
                to "2020-04-01T00:00:00".
            device_ids (list of int, optional): What devices to get site
                accesses for.
            site_ids (list of int, optional): What sites to get site accesses
                for.
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the `SiteAccess`
                schema (See Response Type). By default all fields are
                included.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            list of SiteAccess: Response from the API. A list of site accesses
                matching the filter parameters.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/site-accesses'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'timestampStart': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_start),
            'timestampEnd': APIHelper.when_defined(APIHelper.RFC3339DateTime, timestamp_end),
            'deviceIds': device_ids,
            'siteIds': site_ids,
            'fields': fields,
            'pageSize': page_size,
            'page': page,
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, SiteAccess.from_dictionary)

    def get_site_access(self,
                        site_access_id,
                        use_utc=False):
        """Does a GET request to /site-accesses/{siteAccessId}.

        **(BETA)** Gets a specific site access.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            site_access_id (int): The id of the site access.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            SiteAccess: Response from the API. A single site access object.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/site-accesses/{siteAccessId}'
        _url_path = APIHelper.append_url_with_template_parameters(_url_path, { 
            'siteAccessId': site_access_id
        })
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 404:
            raise APIException('The requested resource could not be found.', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, SiteAccess.from_dictionary)

    def list_device_accesses(self,
                             site_access_id,
                             fields=None,
                             page_size=50,
                             page=1,
                             use_utc=False):
        """Does a GET request to /site-accesses/{siteAccessId}/device-accesses.

        **(BETA)** Gets a list of device accesses belonging to a site access.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            site_access_id (int): The id of the site access.
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the `DeviceAccess`
                schema (See Response Type). By default all fields are
                included.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            list of DeviceAccess: Response from the API. A list of device
                accesses.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/site-accesses/{siteAccessId}/device-accesses'
        _url_path = APIHelper.append_url_with_template_parameters(_url_path, { 
            'siteAccessId': site_access_id
        })
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'fields': fields,
            'pageSize': page_size,
            'page': page,
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 404:
            raise APIException('The requested resource could not be found.', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, DeviceAccess.from_dictionary)

    def get_device_access(self,
                          device_access_id,
                          use_utc=False):
        """Does a GET request to /device-accesses/{deviceAccessId}.

        **(BETA)** Get a single device access by ID.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            device_access_id (int): The id of the device access to get.
            use_utc (bool, optional): Set to true to get timestamps in UTC.

        Returns:
            DeviceAccess: Response from the API. A single device access.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/device-accesses/{deviceAccessId}'
        _url_path = APIHelper.append_url_with_template_parameters(_url_path, { 
            'deviceAccessId': device_access_id
        })
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'useUtc': use_utc
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 404:
            raise APIException('The requested resource could not be found.', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, DeviceAccess.from_dictionary)

    def list_organizations(self):
        """Does a GET request to /organizations.

        **(BETA)** Gets a list of organizations.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Returns:
            list of Organization: Response from the API. A list of
                organizations.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/organizations'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, Organization.from_dictionary)

    def list_personnel(self,
                       fields=None,
                       page_size=50,
                       page=1):
        """Does a GET request to /personnel.

        **(BETA)** Gets a list of personnel.
        _🔐 This endpoint requires the **Plan** endpoint permission._
        _This is a beta feature. Some details might change before it is
        released as a stable version._

        Args:
            fields (list of string, optional): Which fields to include in the
                response. Valid fields are those defined in the `Personnel`
                schema (See Response Type). By default all fields are
                included.
            page_size (int, optional): The number of items to return per
                page.
            page (int, optional): Which page to return when the number of
                items exceed the page size.

        Returns:
            list of Personnel: Response from the API. A list of personnel
                matching the filter parameters.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/personnel'
        _query_builder = Configuration.get_base_uri()
        _query_builder += _url_path
        _query_parameters = {
            'fields': fields,
            'pageSize': page_size,
            'page': page
        }
        _query_builder = APIHelper.append_url_with_query_parameters(_query_builder,
            _query_parameters, Configuration.array_serialization)
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.http_client.get(_query_url, headers=_headers)
        CustomHeaderAuth.apply(_request)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise ProblemDetailsException('The request cannot be fulfilled due to bad syntax.', _context)
        elif _context.response.status_code == 401:
            raise APIException('The request is missing a valid API key. ', _context)
        elif _context.response.status_code == 403:
            raise APIException('One of the following: * The API key does not authorize access to the requested endpoint because of a missing endpoint permission. * The API key does not authorize access to the requested data. Devices, sites or data signals can be limited. ', _context)
        elif _context.response.status_code == 405:
            raise APIException('The HTTP method is not allowed for the endpoint.', _context)
        elif _context.response.status_code == 429:
            raise ProblemDetailsException('The API key has been used in too many requests in a given amount of time. The following headers will be set in the response: * `X-Rate-Limit-Limit` – The rate limit period (for example   "1m", "12h", or "1d"). * `X-Rate-Limit-Remaining` – The remaining number of requests   for this period. * `X-Rate-Limit-Reset` – The UTC timestamp string (in ISO 8601   format) when the remaining number of requests resets.  The limit is currently 1,000 requests/minute per API key and IP address. ', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, Personnel.from_dictionary)



# -*- coding: utf-8 -*-

"""
    greenbyteapi

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

import jsonpickle
import dateutil.parser
from .controller_test_base import ControllerTestBase
from ..test_helper import TestHelper
from greenbyteapi.api_helper import APIHelper


class StatusesControllerTests(ControllerTestBase):

    @classmethod
    def setUpClass(cls):
        super(StatusesControllerTests, cls).setUpClass()
        cls.controller = cls.api_client.statuses

    # Gets statuses for multiple devices during the given time period.
    #The timestamps are in the time zone configured in the Greenbyte Platform.
    #Use the useUtc flag to get timestamps in UTC.
    #
    #_🔐 This endpoint requires the **Statuses** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `status.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_statuses(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        category = None
        lost_production_signal_id = 432
        fields = APIHelper.json_deserialize('["deviceId","message","lostProduction"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.get_statuses(device_ids, timestamp_start, timestamp_end, category, lost_production_signal_id, fields, sort_by, sort_asc, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))


    # Gets statuses for multiple devices during the given time period.
    #The timestamps are in the time zone configured in the Greenbyte Platform.
    #Use the useUtc flag to get timestamps in UTC.
    #
    #_🔐 This endpoint requires the **Statuses** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `status.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_statuses_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        timestamp_start = APIHelper.RFC3339DateTime.from_value('2020-01-01T00:00:00Z').datetime
        timestamp_end = APIHelper.RFC3339DateTime.from_value('2020-01-08T00:00:00Z').datetime
        category = None
        lost_production_signal_id = 432
        fields = APIHelper.json_deserialize('["deviceId","message","lostProduction"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.get_statuses(device_ids, timestamp_start, timestamp_end, category, lost_production_signal_id, fields, sort_by, sort_asc, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

    # Gets active statuses for multiple devices.
    #The timestamps are in the time zone configured in the Greenbyte Platform.
    #Use the useUtc flag to get timestamps in UTC.
    #
    #_🔐 This endpoint requires the **Statuses** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `activestatus.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_active_statuses(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        category = None
        lost_production_signal_id = 432
        fields = APIHelper.json_deserialize('["deviceId","message","lostProduction"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.get_active_statuses(device_ids, category, lost_production_signal_id, fields, sort_by, sort_asc, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 200)

        # Test headers
        expected_headers = {}
        expected_headers['content-type'] = 'application/json; charset=utf-8'

        self.assertTrue(TestHelper.match_headers(expected_headers, self.response_catcher.response.headers))


    # Gets active statuses for multiple devices.
    #The timestamps are in the time zone configured in the Greenbyte Platform.
    #Use the useUtc flag to get timestamps in UTC.
    #
    #_🔐 This endpoint requires the **Statuses** endpoint permission._
    #
    #_This request can also be made using the POST method, 
    #with a request to `activestatus.json` and 
    #a JSON request body instead of query parameters._
    #
    def test_test_get_active_statuses_1(self):
        # Parameters for the API call
        device_ids = APIHelper.json_deserialize('[1,2,3]')
        category = None
        lost_production_signal_id = 432
        fields = APIHelper.json_deserialize('["deviceId","message","lostProduction"]')
        sort_by = None
        sort_asc = False
        page_size = 50
        page = 1
        use_utc = False

        # Perform the API call through the SDK function
        result = self.controller.get_active_statuses(device_ids, category, lost_production_signal_id, fields, sort_by, sort_asc, page_size, page, use_utc)

        # Test response code
        self.assertEquals(self.response_catcher.response.status_code, 204)

